package de.uni_passau.fim.PADAS.group3.DataDash.model;

import java.net.URL;
import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.sql.Date;

import jakarta.persistence.Entity;
import jakarta.persistence.EnumType;
import jakarta.persistence.Enumerated;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.GenerationType;
import jakarta.persistence.Id;
import jakarta.persistence.ManyToOne;

@Entity
public class Dataset {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private UUID id;
    
    @Enumerated(EnumType.STRING)
    private Type type;

    private String title;

    private String abst;

    private String description;

    private String author;

    private Date date;

    private float raiting;

    private int votes;

    private int upvotes;

    private URL url;

    private String licence;

    private static final List<String> sortable = Arrays.asList("author", "title", "upvotes", "date");

    @ManyToOne
    private Category categorie;

    public Dataset(String title, String abst, String description, String author, URL url, Category categories, Type type, String licence) {

        this.raiting = 0;
        this.votes = 0;
        this.upvotes = 0;
        setTitle(title);
        setAbst(abst);        
        setDescription(description);        
        setAuthor(author);
        setDate(LocalDate.now());
        setCategorie(categories);        
        setType(type);
        setUrl(url);
        setLicence(licence);
    }

    public Dataset() {
    }


    public String getAbst() {
        return abst;
    }

    public String getAuthor() {
        return author;
    }

    public Category getCategorie() {
        return categorie;
    }

    public LocalDate getDate() {
        return date.toLocalDate();
    }

    public String getDescription() {
        return description;
    }

    public UUID getId() {
        return id;
    }

    public float getRaiting() {
        return raiting;
    }

    public String getTitle() {
        return title;
    }

    public Type getType() {
        return type;
    }

    public int getVotes() {
        return votes;
    }

    public int getUpvotes() {
        return upvotes;
    }

    public URL getUrl() {
        return url;
    }

    public String getLicence() {
        return licence;
    }

    public static List<String> getSort() {
        return sortable;
    }

    public void setAbst(String abst) {
        this.abst = abst.substring(0, Math.min(abst.length(), 100));
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public void setCategorie(Category categories) {
        this.categorie = categories;
    }
    
    public void setDate(LocalDate localDate) {
        this.date = Date.valueOf(localDate);
    }
    
    public void setDescription(String description) {
        this.description = description;
    }

    public void setUrl(URL url) {
       this.url = url;
    }
 
    public void setTitle(String title) {
        this.title = title.substring(0, Math.min(title.length(), 50));
    }

    public void setType(Type type) {
        this.type = type;
    }
    
    public void vote(int stars) {
        raiting = (raiting*votes + stars) / (++votes);
    }

    public void upvote() {
        upvotes++;
    }

    public void downvote() {
        upvotes--;
    }

    public void setLicence(String licence) {
        this.licence = licence;
    }

}
